@@ -75,6 +75,8 @@
 #define SHOW_SCREEN_KEY	"/apps/fast-user-switch-applet/show_screen_item"
 #define ACTIVE_ONLY_KEY	"/apps/fast-user-switch-applet/show_active_users_only"
 
+#define LOCKDOWN_DIR    "/desktop/gnome/lockdown"
+#define LOCKDOWN_KEY    LOCKDOWN_DIR "/disable_user_switching"
 
 /* ******************** *
  *  Private Structures  *
@@ -99,6 +101,7 @@ typedef struct _FusaAppletData
 
   guint client_notify_applet_id;
   guint client_notify_global_id;
+  guint client_notify_lockdown_id;
   guint user_notify_id;
   GQuark user_menu_item_quark;
   gint8 pixel_size;
@@ -185,6 +188,10 @@ static void client_notify_global_func (GConfClient *client,
 				       guint        cnxn_id,
 				       GConfEntry  *entry,
 				       gpointer     data);
+static void client_notify_lockdown_func (GConfClient *client,
+                                       guint        cnxn_id,
+                                       GConfEntry  *entry,
+                                       gpointer     data);
 
 /* Child Widget Callbacks */
 static void     prefs_radio_toggled_cb        (GtkToggleButton *button,
@@ -193,9 +200,6 @@ static void     prefs_xnest_check_toggled_cb  (GtkToggleButton *button,
 					       gpointer         data);
 static void     prefs_lock_check_toggled_cb   (GtkToggleButton *button,
 					       gpointer         data);
-static void     about_url_hook                (GtkAboutDialog  *dialog,
-					       const gchar     *url,
-					       gpointer         data);
 static gboolean menubar_button_press_event_cb (GtkWidget       *menubar,
 					       GdkEventButton  *event,
 					       gpointer         data);
@@ -205,6 +209,8 @@ static gboolean menubar_expose_event_cb       (GtkWidget       *menubar,
 static void     menu_style_set_cb             (GtkWidget       *menu,
 					       GtkStyle        *old_style,
 					       gpointer         data);
+static gboolean menu_expose_cb                (GtkWidget       *menu,
+                                               gpointer         data);
 static void     menuitem_destroy_cb           (GtkWidget       *item,
 					       gpointer         data);
 static void     menuitem_style_set_cb         (GtkWidget       *menuitem,
@@ -253,7 +259,6 @@ static void      fusa_applet_data_free        (FusaAppletData *adata);
  * ****************** */
 
 static GtkTooltips *tooltips = NULL;
-static GtkWidget *about = NULL;
 static GnomeProgram *program = NULL;
 
 
@@ -324,6 +329,7 @@ applet_fill_cb (PanelApplet *applet,
 			   "widget \"*.fast-user-switch-menubar\" style \"fast-user-switch-menubar-style\"\n"
 			   "widget \"*.fast-user-switch-applet\" style \"fast-user-switch-applet-style\"\n");
       gtk_window_set_default_icon_name ("stock_people");
+      g_set_application_name (_("Fast User Switch Applet"));
     }
 
   adata = g_new0 (FusaAppletData, 1);
@@ -485,10 +491,20 @@ applet_fill_cb (PanelApplet *applet,
   gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem), adata->menu);
   g_signal_connect (adata->menu, "style-set",
 		    G_CALLBACK (menu_style_set_cb), adata);
+  g_signal_connect (adata->menu, "show",
+		    G_CALLBACK (menu_expose_cb), adata);
   gtk_widget_show (adata->menu);
 
   active_only = gconf_client_get_bool (adata->client, ACTIVE_ONLY_KEY, NULL);
 
+  /* This next part populates the list with all the users we currently know
+   * about. For almost all cases, this is the empty list, because we're
+   * asynchronous, and the data hasn't come back from the callback saying who
+   * the users are yet. However, if someone has two FUSAs on their toolbars (why,
+   * I have no freaking idea, but bear with me here), the menu of the second
+   * one to be initialised needs to be filled in from the start rather than
+   * depending on getting data from the callback like the first one.
+   */
   users = fusa_manager_list_users (adata->manager);
   while (users)
     {
@@ -521,6 +537,7 @@ applet_fill_cb (PanelApplet *applet,
 
       users = g_slist_delete_link (users, users);
     }
+
   g_signal_connect (adata->manager, "user-added",
 		    G_CALLBACK (manager_user_added_cb), adata);
 
@@ -588,10 +605,21 @@ applet_fill_cb (PanelApplet *applet,
     gconf_client_notify_add (adata->client, GLOBAL_DIR,
 			     client_notify_global_func, adata, NULL, NULL);
 
+  adata->client_notify_lockdown_id =
+    gconf_client_notify_add (adata->client, LOCKDOWN_KEY,
+                             client_notify_lockdown_func, adata, NULL, NULL);
+
   adata->items = g_slist_sort_with_data (adata->items,
 					 sort_menu_comparedatafunc, adata);
 
-  gtk_widget_show (GTK_WIDGET (applet));
+  if (gconf_client_get_bool (adata->client, LOCKDOWN_KEY, NULL))
+    {
+      gtk_widget_hide (GTK_WIDGET (applet));
+    }
+  else
+    {
+      gtk_widget_show (GTK_WIDGET (applet));
+    }
 
   return TRUE;
 }
@@ -769,12 +797,8 @@ applet_size_allocate_cb (GtkWidget     *widget,
   PanelAppletOrient orient;
   gint item_border;
   gint pixel_size;
-#if GTK_CHECK_VERSION (2,7,0)
   gdouble text_angle = 0.0;
   GtkPackDirection pack_direction = GTK_PACK_DIRECTION_LTR;
-#else
-  PangoEllipsizeMode ellipsize_mode;
-#endif
 
   adata = data;
   children = gtk_container_get_children (GTK_CONTAINER (adata->menubar));
@@ -792,42 +816,28 @@ applet_size_allocate_cb (GtkWidget     *widget,
     case PANEL_APPLET_ORIENT_DOWN:
       gtk_widget_set_size_request (top_item, -1, allocation->height);
       pixel_size = allocation->height - item_border;
-#if GTK_CHECK_VERSION (2,7,0)
       pack_direction = GTK_PACK_DIRECTION_RTL;
       text_angle = 0.0;
-#else
-      ellipsize_mode = PANGO_ELLIPSIZE_NONE;
-#endif /* HAVE_GTK28 */
       break;
     case PANEL_APPLET_ORIENT_LEFT:
       gtk_widget_set_size_request (top_item, allocation->width, -1);
       pixel_size = allocation->width - item_border;
-#if GTK_CHECK_VERSION (2,7,0)
       pack_direction = GTK_PACK_DIRECTION_TTB;
       text_angle = 270.0;
-#else
-      ellipsize_mode = PANGO_ELLIPSIZE_END;
-#endif /* HAVE_GTK28 */
       break;
     case PANEL_APPLET_ORIENT_RIGHT:
       gtk_widget_set_size_request (top_item, allocation->width, -1);
       pixel_size = allocation->width - item_border;
-#if GTK_CHECK_VERSION (2,7,0)
       pack_direction = GTK_PACK_DIRECTION_TTB;
       text_angle = 90.0;
-#else
-      ellipsize_mode = PANGO_ELLIPSIZE_END;
-#endif /* HAVE_GTK28 */
       break;
     default:
       g_assert_not_reached ();
       break;
     }
 
-#if GTK_CHECK_VERSION (2,7,0)
   gtk_menu_bar_set_pack_direction (GTK_MENU_BAR (adata->menubar),
                                    pack_direction);
-#endif /* HAVE_GTK28 */
 
   if (GTK_IS_IMAGE (adata->imglabel))
     {
@@ -836,13 +846,7 @@ applet_size_allocate_cb (GtkWidget     *widget,
     }
   else
     {
-#if GTK_CHECK_VERSION (2,7,0)
       gtk_label_set_angle (GTK_LABEL (adata->imglabel), text_angle);
-#else
-      gtk_label_set_ellipsize (GTK_LABEL (adata->imglabel),
-                               ellipsize_mode);
-#endif
-
     }
 }
 
@@ -1119,13 +1123,14 @@ about_cb (BonoboUIComponent *ui_container,
 {
   static const gchar *authors[] = {
     "James M. Cape <jcape@ignore-your.tv>",
+    "Thomas Thurman <thomas@thurman.org.uk>",
     NULL
   };
   static const gchar *artists[] = {
     "Hopefully, the users themselves",
     NULL
   };
-  static const gchar *license[] = {
+  static gchar *license[] = {
     N_("The Fast User Switch Applet is free software; you can redistribute it and/or modify "
        "it under the terms of the GNU General Public License as published by "
        "the Free Software Foundation; either version 2 of the License, or "
@@ -1141,48 +1146,25 @@ about_cb (BonoboUIComponent *ui_container,
   };
   gchar *license_i18n;
 
-  if (about)
-    {
-      gtk_window_present (GTK_WINDOW (about));
-      return;
-    }
-
-  about = gtk_about_dialog_new ();
-  g_object_add_weak_pointer (G_OBJECT (about), (gpointer *) &about);
-  gtk_about_dialog_set_logo_icon_name (GTK_ABOUT_DIALOG (about),
-				       "stock_people");
-  gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (about),
-			     "Fast User Switch Applet");
-  gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (about), VERSION);
-  gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (about),
-				 _("A menu to quickly switch between users."));
-  gtk_about_dialog_set_copyright (GTK_ABOUT_DIALOG (about),
-				  "Copyright \xc2\xa9 2004-2005 James M. Cape.");
-
-  gtk_about_dialog_set_url_hook (about_url_hook, NULL, NULL);
-  gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (about),
-				"http://ignore-your.tv/fusa/");
-
-  gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (about), authors);
-  gtk_about_dialog_set_artists (GTK_ABOUT_DIALOG (about), artists);
-  /* Translators: This is a special message that shouldn't be translated
-   * literally. It is used in the about box to give credits to
-   * the translators.
-   * Thus, you should translate it to your name and email address.
-   * You should also include other translators who have contributed to
-   * this translation; in that case, please write each of them on a separate
-   * line seperated by newlines (\n).
-   */
-  gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG (about),
-					   _("translator-credits"));
   license_i18n = g_strjoinv ("\n\n", license);
-  gtk_about_dialog_set_license (GTK_ABOUT_DIALOG (about), license_i18n);
+
+  gtk_show_about_dialog(NULL,
+      "version", VERSION,
+      "copyright", "Copyright \xc2\xa9 2004-2005 James M. Cape.\n"
+                   "Copyright \xc2\xa9 2006 Thomas Thurman.",
+      "comments", _("A menu to quickly switch between users."),
+      "authors", authors,
+      "artists", artists,
+      "license", license_i18n,
+      "wrap-license", TRUE,
+      "translator-credits", _("translator-credits"),
+      "logo-icon-name", "stock_people",
+      "website", "http://ignore-your.tv/fusa/",
+      "website-label", "Fast User Switch Applet Website",
+      NULL
+   );
+
   g_free (license_i18n);
-
-  g_object_add_weak_pointer (G_OBJECT (about),
-			     (gpointer *) &about);
-
-  gtk_window_present (GTK_WINDOW (about));
 }
 
 
@@ -1265,6 +1247,36 @@ manager_user_added_cb (FusaManager *manager,
   sort_menu (adata);
 }
 
+static void
+client_notify_lockdown_func (GConfClient *client,
+                             guint        cnxn_id,
+                             GConfEntry  *entry,
+                             gpointer     data)
+{
+  FusaAppletData *adata;
+  GConfValue *value;
+  const gchar *key;
+             
+  adata = data;
+  value = gconf_entry_get_value (entry);
+  key = gconf_entry_get_key (entry);
+             
+  if (!value || !key)
+    return;
+             
+  if (strcmp (key, LOCKDOWN_KEY) == 0)
+    {
+      if (gconf_value_get_bool (value))
+        {
+          gtk_widget_hide ( GTK_WIDGET (adata->applet));
+        }
+      else
+        {
+          gtk_widget_show ( GTK_WIDGET (adata->applet));
+        }
+    }
+}            
+
 static void
 user_displays_changed_cb (FusaUser *user,
 			  gpointer  data)
@@ -1556,14 +1568,6 @@ prefs_lock_check_toggled_cb (GtkToggleButton *button,
 			       gtk_toggle_button_get_active (button), NULL);
 }
 
-static void
-about_url_hook (GtkAboutDialog *dialog,
-		const gchar    *url,
-		gpointer        data)
-{
-  gnome_vfs_url_show (url);
-}
-
 /*
  * gnome-panel/applets/wncklet/window-menu.c:window_filter_button_press()
  * 
@@ -1621,6 +1625,45 @@ menubar_expose_event_cb (GtkWidget      *widget,
   return FALSE;
 }
 
+/* When the user opens the menu of users, menu_expose_cb gets called.
+ * That then calls GDM to ask it which users have current logins,
+ * and when it has responded and we've updated the relevant data
+ * structures, it calls this callback.
+ *
+ * All we have to do here is repaint the menu, since the menu items
+ * are clever and will add or remove their check marks according to
+ * whether their users have any current logins.
+ */
+static void
+redraw_menu_cb (FusaManager  *manager,
+    FusaDisplay  *display,
+    const GError *error,
+    gpointer      data)
+{
+  FusaAppletData *adata = data;
+      
+  gtk_widget_queue_draw (adata->menu);
+}
+
+
+/* Called every time the menu is displayed (and also for some reason
+ * immediately it's created, which does no harm). All we have to do
+ * here is kick off a request to GDM to let us know which users are
+ * logged in, so we can display check marks next to their names.
+ */
+static gboolean
+menu_expose_cb (GtkWidget *menu,
+	          gpointer   data)
+{
+  FusaAppletData *adata = data;
+
+  fusa_manager_request_update_displays (adata->manager,
+      redraw_menu_cb,
+      data);
+
+  return FALSE;
+}
+
 static void
 menu_style_set_cb (GtkWidget *menu,
 		   GtkStyle  *old_style,
@@ -2127,6 +2170,7 @@ show_error_dialog (FusaAppletData *adata,
 
       dialog = glade_xml_get_widget (xml, "error_dialog");
       gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
 
       if (is_fatal)
 	bonobo_main_quit ();
@@ -2176,7 +2220,7 @@ static void
 sort_menu (FusaAppletData *adata)
 {
   GSList *items;
-  guint n_items, n_rows, n_cols, row, column;
+  guint n_items, n_rows, n_cols, row, column, count;
   gint screen_height;
 
   if (!gtk_widget_has_screen (adata->menu))
@@ -2204,6 +2248,7 @@ sort_menu (FusaAppletData *adata)
 
   row = 0;
   column = 0;
+  count = 0;
   items = adata->items;
   while (items)
     {
@@ -2217,6 +2262,13 @@ sort_menu (FusaAppletData *adata)
 	      row = 0;
 	      column++;
 	    }
+
+          /* Just re-attaching them doesn't alter the order you get them
+           * in when you move through with the arrow keys, though; we
+           * have to set that explicitly.
+           */
+          gtk_menu_reorder_child (GTK_MENU (adata->menu),
+              items->data, count++);
 	}
 
       items = items->next;
